/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.security.ssh

import org.apache.sshd.server.session.ServerSession
import org.artifactory.api.security.UserInfoBuilder
import org.artifactory.storage.security.service.UserGroupStoreService
import org.bouncycastle.jcajce.provider.asymmetric.rsa.BCRSAPublicKey
import spock.lang.Specification

import java.security.spec.RSAPublicKeySpec

/**
 * @author Noam Y. Tenne
 */
class PublicKeyAuthenticatorSpec extends Specification {

    def 'Authenticate public key'() {
        def userGroupStoreService = Mock(UserGroupStoreService)
        def serverSession = Mock(ServerSession)
        def authenticator = new PublicKeyAuthenticator(userGroupStoreService)

        def publicKey = new BCRSAPublicKey(new RSAPublicKeySpec(1.toBigInteger(), 3.toBigInteger()));

        when:
        def result = authenticator.authenticate('git', publicKey, serverSession)

        then:
        result
        1 * userGroupStoreService.findUserByProperty('sshPublicKey', 'ssh-rsa AAAAB3NzaC1yc2EAAAABAwAAAAEB') >> {
            return new UserInfoBuilder('jim').build()
        }
        1 * serverSession.setAttribute(_ as UsernameAttributeKey, 'jim')
    }

    def 'Authenticate public key and find no user '() {
        def userGroupStoreService = Mock(UserGroupStoreService)
        def serverSession = Mock(ServerSession)
        def authenticator = new PublicKeyAuthenticator(userGroupStoreService)

        def publicKey = new BCRSAPublicKey(new RSAPublicKeySpec(1.toBigInteger(), 3.toBigInteger()));

        when:
        def result = authenticator.authenticate('git', publicKey, serverSession)

        then:
        !result
        1 * userGroupStoreService.findUserByProperty('sshPublicKey', 'ssh-rsa AAAAB3NzaC1yc2EAAAABAwAAAAEB') >> {
            return null
        }
    }
}
