/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.addon.ha;

import org.artifactory.addon.Addon;
import org.artifactory.addon.ha.message.HaMessage;
import org.artifactory.addon.ha.message.HaMessageTopic;
import org.artifactory.addon.ha.semaphore.SemaphoreWrapper;
import org.artifactory.security.props.auth.CacheWrapper;
import org.artifactory.security.props.auth.CacheWrapper.CacheConfig;
import org.artifactory.storage.db.servers.model.ArtifactoryServer;

import java.util.List;

/**
 * @author mamo, gidi, fred
 */
public interface HaCommonAddon extends Addon {

    /**
     * Response header for an HA Artifactory Node
     */
    String ARTIFACTORY_NODE_ID = "X-Artifactory-Node-Id";

    /**
     * Server ID in case of a non HA Artifactory node
     */
    String ARTIFACTORY_PRO = "Artifactory";

    String SUPPORT_BUNDLE_SEMAPHORE_NAME = "supportBundleSemaphore";
    String STATS_SEMAPHORE_NAME = "flushStatsSemaphore";
    String STATS_REMOTE_SEMAPHORE_NAME = "flushRemoteStatsSemaphore";
    String INDEXING_SEMAPHORE_NAME = "indexingSemaphore";
    String INDEX_MARKED_ARCHIVES_SEMAPHORE_NAME = "indexMarkedArchivesSemaphore";
    int DEFAULT_SEMAPHORE_PERMITS = 1;
    // we block concurrent executions to prevent race condition
    // between delete(async) and list(),
    //
    // also there is no justification to perform concurrent execution
    // as we support taking several thread dumps with or without interval
    int SUPPORT_BUNDLE_SEMAPHORE_INITIAL_PERMITS = 1;

    /**
     * determines if HA is enabled.
     * <p>that is {@link org.artifactory.common.ArtifactoryHome#ARTIFACTORY_HA_NODE_PROPERTIES_FILE} exists and license type
     * is HA or Trial
     *
     * @return {@code true} if the current Artifactory instance is HA enabled
     */
    boolean isHaEnabled();

    /**
     * @return {@code true} if HA is enabled and activated, and current Artifactory instance is the primary.
     *         <p>is HA is <b>not enabled</b>, return true
     */
    boolean isPrimary();

    /**
     * @return {@code true} if HA is configured.
     */
    boolean isHaConfigured();

    //todo move into HaAddon, should not be a common interface
    void notify(HaMessageTopic haMessageTopic, HaMessage haMessage);

    /**
     * @return A unique hostId for a Pro/OSS installation, a hashed token for an HA node
     */
    String getHostId();

    SemaphoreWrapper getSemaphore(String semaphoreName);

    <K, V> CacheWrapper<K, V> getCache(String cacheName, CacheConfig cacheConfig);

    void shutdown();

    List<ArtifactoryServer> getAllArtifactoryServers();

    boolean deleteArtifactoryServer(String id);

    boolean artifactoryServerHasHeartbeat(ArtifactoryServer artifactoryServer);

    String getCurrentMemberServerId();


    void propagateDebianReindexAll(ArtifactoryServer server, String repoKey, boolean async, boolean writeProps);

    void propagateOpkgReindexAll(ArtifactoryServer server, String repoKey, boolean async, boolean writeProps);

    List propagateTrafficCollector(long startLong, long endLong, List<String> ipsToFilter,
            List<ArtifactoryServer> servers, Class clazz) ;

    <T> List<T> propagateTasksList(List<ArtifactoryServer> servers, Class<T> clazz);

    void forceOptimizationOnce();
}
