/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.converters.helpers;

import org.artifactory.storage.db.properties.model.DbProperties;
import org.artifactory.storage.db.properties.service.ArtifactoryCommonDbPropertiesService;
import org.artifactory.version.ArtifactoryVersion;

import java.util.Date;

/**
 * @author Gidi Shabat
 */
public class MockDbPropertiesService implements ArtifactoryCommonDbPropertiesService {
    private ArtifactoryVersion version;
    private long release;
    private DbProperties dbProperties;

    public MockDbPropertiesService(ArtifactoryVersion version, long release) {
        this.version = version;
        this.release = release;
    }

    @Override
    public void updateDbProperties(DbProperties dbProperties) {
        this.dbProperties = dbProperties;
    }

    @Override
    public DbProperties getDbProperties() {
        if (version == null) {
            return null;
        } else {
            return new DbProperties(new Date().getTime(), version.getValue(), (int) version.getRevision(), release);
        }
    }

    @Override
    public boolean isDbPropertiesTableExists() {
        return version != null && ArtifactoryVersion.v310.beforeOrEqual(version);
    }

    public boolean isUpdateDbPropertiesHasBeenCalled() {
        return dbProperties != null;
    }
}
