/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.storage.fs.lock.map;


import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @author Shay Yaakov
 */
public class JVMLockingMap<T> implements LockingMap<T> {

    private final Map<T, T> locks = new HashMap<>();

    public static void main(String[] args) {
        long l = TimeUnit.MILLISECONDS.toMillis(5000);
        System.out.println(l);
    }

    @Override
    public boolean tryAddAndLock(T key, long timeout, TimeUnit timeUnit) throws InterruptedException {
        long startTime = System.currentTimeMillis();
        while (true) {
            synchronized (locks) {
                if (locks.containsKey(key)) {
                    if (timeout == 0) {
                        return false;
                    }
                    long userTimeout = timeUnit.toMillis(timeout);
                    locks.wait(userTimeout);
                    long endTime = System.currentTimeMillis();
                    if (endTime - startTime >= userTimeout) {
                        return false;
                    }
                } else {
                    locks.put(key, key);
                    return true;
                }
            }
        }
    }

    @Override
    public void removeAndUnlock(T key) {
        synchronized (locks) {
            locks.remove(key);
            locks.notifyAll();
        }
    }

    @Override
    public void removeAndForceUnlock(T key) {
        removeAndUnlock(key);
    }

    @Override
    public boolean isLocked(T key) {
        synchronized (locks) {
            return locks.containsKey(key);
        }
    }

    public int size() {
        synchronized (locks) {
            return locks.size();
        }
    }

    public Set<T> keySet() {
        synchronized (locks) {
            return locks.keySet();
        }
    }
}
