package org.artifactory.storage.db.util.querybuilder;

import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.artifactory.storage.db.util.querybuilder.QueryBuilderUtils.addButLimit;
import static org.testng.Assert.*;

/**
 * @author Yinon Avraham
 * Created on 09/08/2016.
 */
@Test
public class QueryBuilderUtilsTest {

    @Test(dataProvider = "provideAddButLimit")
    public void testAddButLimit(long a, long b, long max, long expected) {
        assertEquals(addButLimit(a, b, max), expected);
    }

    @DataProvider
    public static Object[][] provideAddButLimit() {
        return new Object[][] {
                {0,0,0,0},
                {0,5,10,5},
                {5,0,10,5},
                {300,200,1000,500},
                {0,10,5,5},
                {10,0,5,5},
                {Long.MAX_VALUE,Long.MAX_VALUE,Long.MAX_VALUE,Long.MAX_VALUE},
                {Long.MAX_VALUE-10,Long.MAX_VALUE-10,Long.MAX_VALUE,Long.MAX_VALUE},
                {Long.MAX_VALUE,Long.MAX_VALUE,10,10},
        };
    }

    @Test(dataProvider = "provideAddButLimitIllegalArgs", expectedExceptions = {AssertionError.class})
    public void testAddButLimitIllegalArgs(long a, long b, long max) {
        addButLimit(a, b, max);
    }

    @DataProvider
    public static Object[][] provideAddButLimitIllegalArgs() {
        return new Object[][] {
                {-1,0,0},
                {0,-2,0},
                {0,0,-3},
                {Long.MIN_VALUE,0,0},
                {0,Long.MIN_VALUE,0},
                {0,0,Long.MIN_VALUE}
        };
    }

}