/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.support.core.collectors.config;

/**
 * Helper class for ConfigDescriptorCollector and ConfigFilesCollector
 *
 * @author nadavy
 */
public class ConfigCollectorHelper {
    public static final boolean isXml = true;

    private static final String[] XML_SCRUB_FIELD = {
            "password",
            "refreshToken",
            "secret",
            "managerPassword",
            "passphrase"};

    private static final String[] PROPERTY_SCRUB_FIELD = {
            "password"};


    /**
     * Scrubs a string from user data (if exists)
     *
     * @param dataToScrub String line to check for data to scrubb
     * @param isXml       true if string from xml, false is a config file string.
     * @return scrubbed line
     */
    public static String scrubDescriptorLine(String dataToScrub, boolean isXml) {
        String[] fields = (isXml) ? XML_SCRUB_FIELD : PROPERTY_SCRUB_FIELD;
        for (String scrub : fields) {
            dataToScrub = dataToScrub.replaceAll(createExposedPatternRegEx(scrub, isXml), "");
        }
        return dataToScrub;
    }


    /**
     * Build RegEx of unscrubbed data
     *
     * @param toScrub Field to scrub
     * @param isXml   Is field type is xml
     * @return RegEx string
     */
    private static String createExposedPatternRegEx(String toScrub, boolean isXml) {
        if (isXml) {
            return "\\s*<" + toScrub + ">.+</" + toScrub + ">";
        }
        return toScrub + "=.+";
    }
}
