import EVENTS from '../../../../constants/artifacts_events.constants';
import DICTIONARY from './../../constants/artifact_general.constant';
import TOOLTIP from '../../../../constants/artifact_tooltip.constant';
const infoPropertyBlacklist = ['kbComponentId', 'kbReleaseId', 'catalogComponent', 'compponentLink'];

class jfGovernanceController {

    constructor($scope, GovernanceDao, JFrogEventBus, JFrogGridFactory, JFrogNotifications) {
        this.$scope = $scope;
        this.GovernanceDao = GovernanceDao;
        this.artifactoryGridFactory = JFrogGridFactory;
        this.artifactoryNotifications = JFrogNotifications;
        this.JFrogEventBus = JFrogEventBus;
        this.DICTIONARY = DICTIONARY.governance;
        this.governanceGridOptions = null;
        this.TOOLTIP = TOOLTIP.builds;

        this._init();
    }

    _init() {
        this._createGrid();
        this._getData();
        this._registerEvents();
    }

    _getData() {
        //Temp fix for preventing fetching data for non-file nodes (occurred when pressing "Artifacts" on sidebar)
        if (!this.currentNode.data.path) {
            return;
        }

        this.GovernanceDao.fetch({
            "repoKey": this.currentNode.data.repoKey,
            "path": this.currentNode.data.path
        }).$promise
            .then((data) => {
                if (data.warn) {
                    this.governanceData = {feedbackMsg: data};
                } else {
                    this.governanceData = data;
                    this.governanceGridOptions.setGridData(data.vulnerabilities || []);
                }

            }).catch((err)=>{
                    //console.log(err);
                    if (err.data && err.data.feedbackMsg && err.data.feedbackMsg.error) {
                        this.artifactoryNotifications.create({error: err.data.feedbackMsg.error});
                        this.governanceData = err.data;
                    }
                });
    }

    _createGrid() {
        this.governanceGridOptions = this.artifactoryGridFactory.getGridInstance(this.$scope)
            .setRowTemplate('default')
            .setColumns(this._getColumns());
    }

    _getColumns() {
        return [
            {
                name: "Artifact ID",
                displayName: 'Artifact ID',
                field: "artifactID",
                allowGrouping: true,
            },
            {
                name: "Name",
                displayName: "Name",
                field: "name",
                allowGrouping: true,
            },
            {
                name: "Severity",
                displayName: "Severity",
                field: "severity",
                allowGrouping: true,
            },
            {
                name: "Description",
                displayName: "Description",
                field: "description",
                allowGrouping: true,
            }
        ]
    }

    _registerEvents() {
        let self = this;
        this.JFrogEventBus.registerOnScope(this.$scope, EVENTS.TAB_NODE_CHANGED, (node) => {
            if (this.currentNode != node) {
                this.currentNode = node;
                self._getData();
            }
        });
    }

    propertyShouldDisplay(property) {
        return !_.contains(infoPropertyBlacklist, property);
    }

    updateComponentId(componentId) {
        this.GovernanceDao.update({
            "repoKey": this.currentNode.data.repoKey,
            "path": this.currentNode.data.path,
            "componentId": componentId,
            "origComponentId": this.governanceData.info.componentId
        }).$promise.then(() => this._getData());
        this.editMode = false;
    }

    cancelEditMode($event) {
        let element = $($event.toElement);
        if (!element.hasClass('fa') && element.prop('tagName') !== 'INPUT') {
            this.editMode = false;
        }
    }

}
export function jfGovernance() {
    return {
        restrict: 'EA',
        controller: jfGovernanceController,
        controllerAs: 'jfGovernance',
        bindToController: true,
        templateUrl: 'states/artifacts/jf_artifact_info/info_tabs/jf_governance.html',
        scope: {
            currentNode: '='
        }
    }
}