package org.artifactory.ui.rest.service.artifacts.browse.treebrowser.tabs.composer;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.composer.ComposerAddon;
import org.artifactory.addon.composer.ComposerMetadataInfo;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.repo.InternalRepoPathFactory;
import org.artifactory.repo.RepoPath;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.ui.rest.model.artifacts.browse.treebrowser.tabs.composer.ComposerArtifactInfo;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

/**
 * @author Shay Bagants
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class ComposerViewService implements RestService {

    @Override
    public void execute(ArtifactoryRestRequest request, RestResponse response) {
        ComposerArtifactInfo composerArtifactInfo = (ComposerArtifactInfo) request.getImodel();
        // Fetch npm meta data
        fetchComposerMetadata(request, response, composerArtifactInfo);
    }

    private void fetchComposerMetadata(ArtifactoryRestRequest artifactoryRequest, RestResponse artifactoryResponse,
            ComposerArtifactInfo composerArtifactInfo) {
        AddonsManager addonsManager = ContextHelper.get().beanForType(AddonsManager.class);
        ComposerAddon composerAddon = addonsManager.addonByType(ComposerAddon.class);
        RepoPath repoPath = InternalRepoPathFactory
                .create(composerArtifactInfo.getRepoKey(), composerArtifactInfo.getPath());
        ComposerMetadataInfo composerMetadataInfo = composerAddon.getComposerMetadataInfo(repoPath);
        artifactoryResponse.iModel(composerMetadataInfo);
    }
}
