/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.service.artifacts.browse.treebrowser.tabs.nugetview;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.nuget.UiNuGetAddon;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.nuget.NuMetaData;
import org.artifactory.repo.InternalRepoPathFactory;
import org.artifactory.repo.RepoPath;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.ui.rest.model.artifacts.browse.treebrowser.tabs.nugetinfo.NugetArtifactInfo;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

/**
 * @author Chen Keinan
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class NugetViewService implements RestService {
    @Override
    public void execute(ArtifactoryRestRequest request, RestResponse response) {
        NugetArtifactInfo nugetArtifactInfo = (NugetArtifactInfo) request.getImodel();
        // get nuGet Meta data
        NugetArtifactInfo nuGetMetaData = getNuGetMetaData(nugetArtifactInfo);
        // update response with model data
        if (nuGetMetaData != null) {
            response.iModel(nuGetMetaData);
        }
    }

    /**
     * get  Nuget MetaData model
     *
     * @param nugetArtifactInfo
     * @return nuGet Meta Data model
     */
    private NugetArtifactInfo getNuGetMetaData(NugetArtifactInfo nugetArtifactInfo) {
        String repoKey = nugetArtifactInfo.getRepoKey();
        String path = nugetArtifactInfo.getPath();
        RepoPath repoPath = InternalRepoPathFactory.create(repoKey, path);
        AddonsManager addonsManager = ContextHelper.get().beanForType(AddonsManager.class);
        UiNuGetAddon uiNuGetAddon = addonsManager.addonByType(UiNuGetAddon.class);
        if (uiNuGetAddon != null) {
            NuMetaData nugetSpecMetaData = uiNuGetAddon.getNutSpecMetaData(repoPath);
            nugetArtifactInfo = new NugetArtifactInfo(nugetSpecMetaData);
        }
        return nugetArtifactInfo;
    }
}
